@@ -59,7 +59,6 @@ char buf[256], *p;
 	fflush(stdout);
 
 	flags |= FLAGS_CMD_MODE;
-
 	while((p = fgets(buf, 256, stdin)) != NULL) {
 		cstrip_line(buf);
 		if (*buf)
@@ -74,6 +73,8 @@ char buf[256], *p;
 			break;
 	}
 	if (p == NULL) {
+		clearerr(stdin);
+
 		if (AllConns == NULL) {
 			printf("\n\n");
 			terminal_mode(TERMINAL_COOKED);
@@ -90,6 +91,41 @@ char buf[256], *p;
 	terminal_mode(TERMINAL_RAW);
 }
 
+int put_conn(Conn *c, char kar) {
+int err, saved_errno;
+
+	errno = 0;
+	if (c == NULL) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	/* regain root privs */
+	if (seteuid(0)) {
+		fprintf(stderr, "failed to regain root privs: %s\n", strerror(errno));
+		cmd_exit(NULL);
+		/* unreached */
+	}
+
+	/* simulate terminal input */
+	err = ioctl(c->fd, TIOCSTI, &kar);
+	saved_errno = errno;
+
+	/* drop root privs again */
+	if (seteuid(getuid())) {
+		fprintf(stderr, "failed to drop root privs: %s\n", strerror(errno));
+		cmd_exit(NULL);
+		/* unreached */
+	}
+
+	if (err == -1) {
+		fprintf(stderr, "\nioctl() : %s\n", strerror(saved_errno));
+		errno = saved_errno;
+		return -1;
+	}
+	return 0;
+}
+
 void pconsole(void) {
 int key, typed = 0;
 char kar;
@@ -163,18 +199,14 @@ Conn *c, *c_next;
 			c_next = c->next;
 
 			if (c->fd > 0) {
-				seteuid(0);											/* regain root privs */
-				if (ioctl(c->fd, TIOCSTI, &kar) == -1) {			/* simulate terminal input */
-					seteuid(getuid());								/* drop root privs again */
-					printf("\nioctl() : %s\n", strerror(errno));
+				if (put_conn(c, kar) == -1) {
 					if (c->hostname != NULL)
 						printf("detaching from %s#%s\n", c->hostname, c->dev);
 					else
 						printf("detaching from %s\n", c->dev);
 					remove_Conn(c);
 					destroy_Conn(c);
-				} else
-					seteuid(getuid());								/* drop the root privs */
+				}
 			}
 		}
 		if (AllConns == NULL)
